#include "p30f6010a.h"
#include "can.h"
#include <stdio.h>
#include "USART.h"
#include "Text_out.h"


	unsigned char data_length;
	unsigned char txdata[] = {0x4D,0x69,0x63,0x72,0x6F,0x43,0x41,0x4E};
	unsigned char rxdata[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	char filter_number;
	char tx_rx_number;
	unsigned char *data_received = &rxdata[0];
	unsigned int sid=0x554;
	unsigned long int eid=0x55555 ;
		
// ***** main *****
			
int main(void)
{
	// initialize Port
	
	PORTD=0x00EF;
	TRISD=0x0010;
	
	// initialize variables
	
	filter_number = 0;
	tx_rx_number = 0;

	// initialize UART
	
	usart_init();
	text_out ("\r\fCAN-Modul \n\n\r");

	// set request for configuration mode

	CAN1SetOperationMode(CAN_CAPTURE_DIS & CAN_IDLE_STOP & CAN_MASTERCLOCK_1 & CAN_REQ_OPERMODE_CONFIG); 
	while (C1CTRLbits.OPMODE <=3);
	
	// load interupt register
	
	ConfigIntCAN1(CAN_INDI_RXB1_EN & CAN_INDI_TXB1_EN & CAN_INDI_ERR_EN, CAN_INT_ENABLE & CAN_INT_PRI_7);

	// load configuration register

	CAN1Initialize(CAN_SYNC_JUMP_WIDTH1 & CAN_BAUD_PRE_SCALE(2), CAN_WAKEUP_BY_FILTER_DIS & CAN_PHASE_SEG2_TQ(3) & CAN_PHASE_SEG1_TQ(3) & CAN_PROPAGATIONTIME_SEG_TQ(3) & CAN_SEG2_FREE_PROG & CAN_SAMPLE1TIME);	

	// load acceptance filter
	
	CAN1SetFilter(filter_number, CAN_FILTER_SID(0) & CAN_RX_EID_DIS, CAN_FILTER_EID(0));
	
	// load mask filter register

	CAN1SetMask(filter_number, CAN_MASK_SID(0) & CAN_MATCH_FILTER_TYPE, CAN_FILTER_EID(0));
	
	// set transmit & receive mode
	
	CAN1SetTXMode(tx_rx_number, CAN_TX_STOP_REQ & CAN_TX_PRIORITY_LOW);
	CAN1SetRXMode(tx_rx_number, CAN_RXFUL_CLEAR & CAN_BUF0_DBLBUFFER_EN);
	
	// set request for normal mode

	CAN1SetOperationMode(CAN_IDLE_CON & CAN_CAPTURE_DIS & CAN_MASTERCLOCK_1 & CAN_REQ_OPERMODE_NOR);
	while (C1CTRLbits.OPMODE !=0);
		
	// load message id, data -> transmit buffer, set transmit request bit

	data_length = 8;
	CAN1SendMessage((CAN_TX_SID(sid)) & (CAN_TX_EID_DIS) & (CAN_SUB_REM_TX_REQ), (CAN_TX_EID(eid)) & (CAN_REM_TX_REQ), txdata, data_length, tx_rx_number);
	C1TX0CONbits.TXREQ = 0;
	// tx_data -> UART
	
	text_out("Gesendete Daten: ");
	putch(txdata[0]);putch(txdata[1]);putch(txdata[2]);putch(txdata[3]);putch(txdata[4]);putch(txdata[5]);putch(txdata[6]);putch(txdata[7]);
	
	// main loop

	while(1)
	{
		// wait till message is transmitted
	
		while (!CAN1IsTXReady(0));
		
		// clear TXREQ
		
		C1TX0CONbits.TXREQ = 0;
		/ tx_data -> UART
	
		text_out("Gesendete Daten: ");
		putch(txdata[0]);
		putch(txdata[1]);
		putch(txdata[2]);
		putch(txdata[3]);
		putch(txdata[4]);
		putch(txdata[5]);
		putch(txdata[6]);
		putch(txdata[7]);
	
	};
	
	return 0;
}

//***** ISR *****

void __attribute((interrupt, no_auto_psv))_C1Interrupt(void)
{
	// clear interrupt flag
	
	IFS1bits.C1IF = 0;		

	// ISR Receive buffer 1 interrupt
	
	if (C1INTFbits.RX1IF)
	{
		C1INTFbits.RX1IF = 0;
		
		// read received data -> data_array
		
		CAN1ReceiveMessage(data_received, data_length, tx_rx_number);
		C1RX0CONbits.RXFUL = 0;
		
		// rx_data -> UART
		
		text_out("\n\rEmpfangene Daten: ");
		putch(rxdata[0]);putch(rxdata[1]);putch(rxdata[2]);putch(rxdata[3]);putch(rxdata[4]);putch(rxdata[5]);putch(rxdata[6]);putch(rxdata[7]);
	
	// ISR Transmitt Buffer 1 interrupt
	
	if (C1INTFbits.TX1IF)
	{
		C1INTFbits.TX1IF = 0;
		
		text_out("Gesendete Daten: ");
		putch(txdata[0]);
		putch(txdata[1]);
		putch(txdata[2]);
		putch(txdata[3]);
		putch(txdata[4]);
		putch(txdata[5]);
		putch(txdata[6]);
		putch(txdata[7]);
	
	}
		
	}
	
	// ISR Error interrupt
	
	if (C1INTFbits.ERRIF)
	{
		C1INTFbits.ERRIF = 0;
		
		// clear interrupt flags
		
		C1INTFbits.EWARN = 0;
		C1INTFbits.RXWAR = 0;
		C1INTFbits.TXWAR = 0;
		C1INTFbits.RXEP = 0;
		C1INTFbits.TXEP = 0;
		C1INTFbits.TXBO = 0;
		C1INTFbits.RX1OVR = 0;
		C1INTFbits.RX0OVR = 0;
		
		LATDbits.LATD0 = 1;
	}
}

void __attribute((interrupt, no_auto_psv))_U1TXInterrupt(void)
{
		IFS0bits.U1TXIF = 0;
		text_out("\n\rEmpfangene Daten: ");
		putch(rxdata[0]);putch(rxdata[1]);putch(rxdata[2]);putch(rxdata[3]);putch(rxdata[4]);putch(rxdata[5]);putch(rxdata[6]);putch(rxdata[7]);
	
}

void __attribute((interrupt, no_auto_psv))_U1RXInterrupt(void)
{
	IFS0bits.U1RXIF = 0;
	while (U1STAbits.UTXBF);
	U1TXREG = U1RXREG;
}
